"""Tests for jautomate.devices"""

from jautomate.models import Asset, User
import jautomate.devices

# Add serial number and asset tag
SAMPLE_COMP = {
    'serial_number': 'XXXXXXXXXXXX',
    'asset_tag': '028516', }
SAMPLE_MOBILE = {
    'serial_number': 'XXXXXXXXXXXX',
    'asset_tag': '021546',
}


def check_for_computer_extension_attribute(attribute_name: str, value: str, payload: dict) -> bool:
    for item in payload["userAndLocation"]["extensionAttributes"]:
        if item.get("name") == attribute_name and item.get("values")[0] == value:
            return True
    return False


def check_for_mobile_extension_attribute(attribute_name: str, value: str, payload: dict) -> bool:
    for item in payload["updatedExtensionAttributes"]:
        if item.get("name") == attribute_name and item.get("value")[0] == value:
            return True
    return False


def test_build_computer_payload():
    asset = Asset(
        serial_number='123456',
        asset_tag='043521',
        device_type='computer',
        jamf_id='4532',
        user=User(
            first_name='Luke',
            last_name='Skywalker',
            email='skywalker.luke@ccpsstaff.org',
            position='Staff',
        )
    )
    payload = jautomate.devices.build_computer_payload(asset)

    assert payload["general"].get("assetTag") == '043521'
    assert payload["userAndLocation"].get("realname") == 'Luke Skywalker'
    assert payload["userAndLocation"].get("email") == 'skywalker.luke@ccpsstaff.org'
    assert payload["userAndLocation"].get("position") == 'Staff'

    asset = Asset(
        serial_number='123456',
        asset_tag='043521',
        device_type='computer',
        user=User(
            first_name='Ben',
            last_name='Solo',
            email='solo.ben@ccpsstaff.org',
            position='Student',
            grad_year='2034',
            grade='1',
            building='PES',
            homeroom='Organa'
        )
    )
    payload = jautomate.devices.build_computer_payload(asset)

    assert payload["general"].get("assetTag") == '043521'
    assert payload["userAndLocation"].get("realname") == 'Ben Solo'
    assert payload["userAndLocation"].get("email") == 'solo.ben@ccpsstaff.org'
    assert payload["userAndLocation"].get("position") == 'Student'
    assert payload["userAndLocation"].get("buildingId") == '5'
    assert payload["userAndLocation"].get("room") == 'Organa'
    assert check_for_computer_extension_attribute("Grade", "1", payload)
    assert check_for_computer_extension_attribute("GradYear", "2034", payload)


def test_build_mobile_payload():
    asset = Asset(
        serial_number='123456',
        asset_tag='043521',
        device_type='mobile',
        jamf_id='4532',
        user=User(
            first_name='Luke',
            last_name='Skywalker',
            email='skywalker.luke@ccpsstaff.org',
            position='Staff',
        )
    )
    payload = jautomate.devices.build_mobile_payload(asset)

    assert payload["name"] == '043521'
    assert payload["location"].get("realName") == 'Luke Skywalker'
    assert payload["location"].get("emailAddress") == 'skywalker.luke@ccpsstaff.org'
    assert payload["location"].get("room") is None

    asset = Asset(
        serial_number='123456',
        asset_tag='043521',
        device_type='mobile',
        user=User(
            first_name='Ben',
            last_name='Solo',
            email='solo.ben@ccpsstaff.org',
            position='Student',
            grad_year='2034',
            grade='1',
            building='PES',
            homeroom='Organa'
        )
    )
    payload = jautomate.devices.build_mobile_payload(asset)

    assert payload["name"] == '043521'
    assert payload["location"].get("realName") == 'Ben Solo'
    assert payload["location"].get("emailAddress") == 'solo.ben@ccpsstaff.org'
    assert payload["location"].get("room") == 'Organa'
    assert check_for_mobile_extension_attribute("Grade", "1", payload)
    assert check_for_mobile_extension_attribute("GradYear", "2034", payload)

    asset = Asset(
        serial_number='123456',
        asset_tag='043521',
        device_type='mobile',
        user=None
    )
    payload = jautomate.devices.build_mobile_payload(asset)
    print(f"Payload: {payload}")
    assert payload["name"] == '043521'
    assert payload["location"].get("realName") is " "
    assert payload["location"].get("emailAddress") is " "
    assert payload["location"].get("room") is " "


def test_get_computer_record_by_serial_number():
    computer = jautomate.devices.get_computer_record_by_serial_number(
        SAMPLE_COMP.get('serial_number')
    )
    assert isinstance(computer, dict)


def test_get_mobile_device_by_serial_number():
    mobile = jautomate.devices.get_mobile_device_by_serial_number(
        SAMPLE_MOBILE.get('serial_number'))
    assert isinstance(mobile, dict)


def test_get_jamf_id_of_mobile_device():
    jamf_id = jautomate.devices.get_jamf_id_of_mobile_device(SAMPLE_MOBILE.get('serial_number'))
    assert isinstance(jamf_id, str)


def test_update_mobile_device_for_staff():
    asset = Asset(
        serial_number=SAMPLE_MOBILE.get('serial_number'),
        asset_tag=SAMPLE_MOBILE.get('asset_tag'),
        device_type='mobile',
        user=User(
            first_name='Luke',
            last_name='Skywalker',
            email='skywalker.luke@ccpsstaff.org',
            position='Staff',
            building='PES',
        )
    )

    asset.jamf_id = jautomate.devices.get_jamf_id_of_mobile_device(asset.serial_number)

    jautomate.devices.update_device(asset)


def test_update_mobile_device_for_student():
    asset = Asset(
        serial_number=SAMPLE_MOBILE.get('serial_number'),
        asset_tag=SAMPLE_MOBILE.get('asset_tag'),
        device_type='mobile',
        user=User(
            first_name='Ben',
            last_name='Solo',
            email='solo.ben@ccpsstudent.org',
            position='Student',
            grad_year='2034',
            grade='1',
            building='PES',
            homeroom='Organa'
        )
    )

    asset.jamf_id = jautomate.devices.get_jamf_id_of_mobile_device(asset.serial_number)

    jautomate.devices.update_device(asset)


def test_unassign_mobile_device():
    asset = Asset(
        serial_number=SAMPLE_MOBILE.get('serial_number'),
        asset_tag=SAMPLE_MOBILE.get('asset_tag'),
        device_type='mobile',
    )

    asset.jamf_id = jautomate.devices.get_jamf_id_of_mobile_device(asset.serial_number)

    jautomate.devices.update_device(asset)


def test_update_computer_staff():
    asset = Asset(
        serial_number=SAMPLE_COMP.get('serial_number'),
        asset_tag=SAMPLE_COMP.get('asset_tag'),
        device_type='computer',
        user=User(
            first_name='Luke',
            last_name='Skywalker',
            username='skywalker-luke',
            email='skywalker.luke@ccpsstaff.org',
            position='Staff',
            building='FES',
            floor='1st',
            grad_year=' ',
            grade=' ',
            homeroom='  '
        )
    )

    asset.jamf_id = jautomate.devices.get_jamf_id_of_computer(asset.serial_number)

    jautomate.devices.update_device(asset)


def test_update_computer_student():
    asset = Asset(
        serial_number=SAMPLE_COMP.get('serial_number'),
        asset_tag=SAMPLE_COMP.get('asset_tag'),
        device_type='computer',
        user=User(
            first_name='Ben',
            last_name='Solo',
            username='solo-ben',
            email='solo.ben@ccpsstudent.org',
            position='Student',
            grad_year='2034',
            grade='1',
            building='PES',
            homeroom='Organa'
        )
    )

    asset.jamf_id = jautomate.devices.get_jamf_id_of_computer(asset.serial_number)

    jautomate.devices.update_device(asset)


def test_unassign_computer():
    asset = Asset(
        serial_number=SAMPLE_COMP.get('serial_number'),
        asset_tag=SAMPLE_COMP.get('asset_tag'),
        device_type='computer',
    )

    asset.jamf_id = jautomate.devices.get_jamf_id_of_computer(asset.serial_number)

    jautomate.devices.update_device(asset)



"""Functions for devices."""

import datetime
from typing import Dict, List
from jps_api_wrapper.request_builder import (
    ClientError, NotFound, RequestConflict, RequestTimedOut, InvalidDataType
)
from jautomate.api import jamf_p, jamf_c
from jautomate.exceptions import JautomateAssetException, JautomateException
from jautomate.logger import j_logger
from jautomate.models import Asset
from jautomate.actions import get_building_id

# #################################
# Mobile Device functions
# #################################


def get_mobile_device_by_serial_number(serial_number: str) -> Dict:
    try:
        response = jamf_c.get_mobile_device(serialnumber=serial_number)

    except (ClientError, NotFound, RequestConflict, RequestTimedOut, InvalidDataType) as e:
        j_logger.error("%s", e)
        raise JautomateException("Could not get mobile device by serial number from Jamf") from e

    return response


def get_jamf_id_of_mobile_device(serial_number: str) -> str:
    mobile_device = get_mobile_device_by_serial_number(serial_number)

    return str(mobile_device.get('mobile_device').get('general').get('id'))


def build_mobile_payload(asset: Asset) -> Dict:
    payload = {
        "enforceName": True,
        "location": {},
        "updatedExtensionAttributes": [],
    }

    if getattr(asset, 'asset_tag', None):
        payload["name"] = asset.asset_tag
        payload["assetTag"] = asset.asset_tag

    # Unset values if no user is passed, typically during an unassign action
    if asset.user is None:
        payload["location"]["realName"] = " "
        payload["location"]["emailAddress"] = " "
        payload["location"]["username"] = " "
        payload["location"]["room"] = " "
        payload["location"]["buildingId"] = None
        payload["location"]["position"] = " "
        payload["location"]["phoneNumber"] = " "

        payload["updatedExtensionAttributes"].append(
            {
                "name": "Grade",
                "value": None,
            }
        )
        payload["updatedExtensionAttributes"].append(
            {
                "name": "Owner",
                "value": None,
            }
        )
        payload["updatedExtensionAttributes"].append(
            {
                "name": "GradYear",
                "value": None,
            }
        )
        payload["updatedExtensionAttributes"].append(
            {
                "name": "Last Sync",
                "value": [datetime.datetime.now().strftime("%m-%d-%Y %H:%M:%S")],
            }
        )
        return payload

    # Begin checking for user attributes
    if getattr(asset.user, 'full_name', None):
        payload["location"]["realName"] = asset.user.full_name

    if getattr(asset.user, 'email', None):
        payload["location"]["emailAddress"] = asset.user.email

    if getattr(asset.user, 'username', None):
        payload["location"]["username"] = asset.user.username

    if getattr(asset.user, 'position', None):
        payload["location"]["position"] = asset.user.position

    if getattr(asset.user, 'phone_number', None):
        payload["location"]["phoneNumber"] = asset.user.phone_number

    if getattr(asset.user, 'building', None):
        payload["location"]["buildingId"] = get_building_id(asset.user.building)

    if getattr(asset.user, 'homeroom', None):
        payload["location"]["room"] = asset.user.homeroom

    if getattr(asset.user, 'grade', None):
        payload["updatedExtensionAttributes"].append(
            {
                "name": "Grade",
                "value": [asset.user.grade],
            }
        )
    if getattr(asset.user, 'grad_year', None):
        payload["updatedExtensionAttributes"].append(
            {
                "name": "GradYear",
                "value": [asset.user.grad_year],
            }
        )
    if getattr(asset.user, 'owner', None):
        payload["updatedExtensionAttributes"].append(
            {
                "name": "Owner",
                "value": [asset.user.owner],
            }
        )
    # Set Last Sync time
    payload["updatedExtensionAttributes"].append(
        {
            "name": "Last Sync",
            "value": [datetime.datetime.now().strftime("%m-%d-%Y %H:%M:%S")],
        }
    )

    return payload


# #################################
# Computer Device functions
# #################################
def build_computer_extension_attribute(extension_id: str, name: str, values: List) -> Dict:
    if isinstance(values, str):
        values = [values]

    extension_attribute = {
        "definitionId": extension_id,
        "name": name,
        "values": values
    }
    return extension_attribute


def build_computer_payload(asset: Asset) -> Dict:
    """Use entire asset object ^ because there may be additional fields we want to sync later"""

    payload = {
        "general": {
            "assetTag": ""
        },
        "userAndLocation": {
            "extensionAttributes": []
        }
    }

    # Set Asset Tag
    if getattr(asset, 'asset_tag', None):
        payload["general"]["assetTag"] = asset.asset_tag

    # Unset values if no user is passed, typically during an unassign action
    if asset.user is None:

        payload["userAndLocation"]["realname"] = ""
        payload["userAndLocation"]["email"] = ""
        payload["userAndLocation"]["position"] = ""
        payload["userAndLocation"]["phone"] = ""
        payload["userAndLocation"]["buildingId"] = None
        payload["userAndLocation"]["room"] = ""
        payload["userAndLocation"]["username"] = ""
        payload["userAndLocation"]["extensionAttributes"].append(
            build_computer_extension_attribute("3", "GradYear", [])
        )
        payload["userAndLocation"]["extensionAttributes"].append(
            build_computer_extension_attribute("4", "Grade", [])
        )
        payload["userAndLocation"]["extensionAttributes"].append(
            build_computer_extension_attribute(
                "5", "Last Sync", datetime.datetime.now().strftime("%m-%d-%Y %H:%M:%S")))
        payload["userAndLocation"]["extensionAttributes"].append(
            build_computer_extension_attribute("6", "Floor", [])
        )
        return payload

    # Begin checking for user attributes
    if getattr(asset.user, 'full_name', None):
        payload["userAndLocation"]["realname"] = asset.user.full_name

    if getattr(asset.user, 'email', None):
        payload["userAndLocation"]["email"] = asset.user.email

    if getattr(asset.user, 'username', None):
        payload["userAndLocation"]["username"] = asset.user.username

    if getattr(asset.user, 'position', None):
        payload["userAndLocation"]["position"] = asset.user.position

    if getattr(asset.user, 'phone_number', None):
        payload["userAndLocation"]["phone"] = asset.user.phone_number

    if getattr(asset.user, 'building', None):
        payload["userAndLocation"]["buildingId"] = get_building_id(asset.user.building)

    if getattr(asset.user, 'homeroom', None):
        payload["userAndLocation"]["room"] = asset.user.homeroom

    if getattr(asset.user, 'grad_year', None):
        payload["userAndLocation"]["extensionAttributes"].append(
            build_computer_extension_attribute("3", "GradYear", [asset.user.grad_year])
        )
    if getattr(asset.user, 'grade', None):
        payload["userAndLocation"]["extensionAttributes"].append(
            build_computer_extension_attribute("4", "Grade", [asset.user.grade])
        )
    if getattr(asset.user, 'floor', None):
        payload["userAndLocation"]["extensionAttributes"].append(
            build_computer_extension_attribute("6", "Floor", [asset.user.floor])
        )
    # Set last sync because we are modifying the record
    payload["userAndLocation"]["extensionAttributes"].append(
        build_computer_extension_attribute(
            "5", "Last Sync", datetime.datetime.now().strftime("%m-%d-%Y %H:%M:%S")))

    return payload


def get_computer_record_by_serial_number(serial_number: str) -> Dict:
    """
    Gets Jamf record of single computer asset based on asset tag

    Args:   asset_tag (str): Asset tag of computer

    Raises: JautomateException: Raised if no computer records found

    Returns:    List[Dict]: List of dictionaries where each dict is a computer asset
    """

    endpoint_filter = f'hardware.serialNumber==\"{serial_number}\"'

    try:
        response = jamf_p.get_computer_inventories(filter=endpoint_filter)

    except (ClientError, NotFound, RequestConflict, RequestTimedOut, InvalidDataType) as e:
        raise JautomateException("Could not get computer record by serial number from Jamf") from e

    if response["totalCount"] >= 1:
        if response["totalCount"] > 1:
            j_logger.warning(
                "Multiple computer records found with serial number %s. "
                "Only the first record will be returned", serial_number)

        return response["results"][0]

    raise JautomateException(f"Computer with serial number {serial_number} not found in Jamf")


def get_jamf_id_of_computer(serial_number: str) -> str:
    computer = get_computer_record_by_serial_number(serial_number)
    return computer.get('id')


# #########################
# Update Functions
# #########################


def build_update_payload(asset: Asset) -> Dict | None:
    if asset.device_type.value == 'mobile':
        return build_mobile_payload(asset)
    if asset.device_type.value == 'computer':
        return build_computer_payload(asset)
    return None


def get_jamf_id(asset_type: str, serial_number: str) -> str:
    if asset_type == 'mobile':
        return get_jamf_id_of_mobile_device(serial_number)
    if asset_type == 'computer':
        return get_jamf_id_of_computer(serial_number)
    return ''


def update_device(asset: Asset) -> None:
    if asset.jamf_id is None:
        asset.jamf_id = get_jamf_id(asset.device_type.value, asset.serial_number)

    if asset.jamf_id == '':
        raise JautomateAssetException("Jamf ID not set", asset)

    payload = build_update_payload(asset)

    if payload is None:
        raise JautomateAssetException("Update payload not set", asset)

    try:
        if asset.device_type.value == 'computer':
            jamf_p.update_computer_inventory(payload, asset.jamf_id)
            j_logger.info("Computer Asset: %s Ok", asset.asset_tag)

        if asset.device_type.value == 'mobile':
            jamf_p.update_mobile_device(payload, asset.jamf_id)
            j_logger.info("Mobile Asset: %s Ok", asset.asset_tag)

    except (ClientError, NotFound, RequestConflict, RequestTimedOut) as e:
        raise JautomateException("Could not update device in Jamf") from e

    except InvalidDataType as e:
        raise JautomateException("Invalid Data sent to Jamf") from e



"""Exceptions for jautomate"""

from jautomate.models import Asset


class JautomateException(Exception):
    """
    Base Exception for Jautomate
    """


class JautomateAssetException(JautomateException):
    """
    Base Exception for Jautomate
    """

    def __init__(self, message: str, asset: Asset | None = None):
        super().__init__(message)
        self.asset = asset



"""Utility functions for Jautomate"""

import csv
import os
from typing import Dict, List, Union

from jautomate.assets import Asset
from jautomate.logger import j_logger


def get_assets_from_csv(file_path: Union[str, os.PathLike]) -> List[Asset]:
    """
    Imports assets from csv file and stores as Asset Objects.

    Will import from a CSV file of assets using specific keys for the
    information.
    The keys are as follows

    CCPS Tag#
    Serial Number
    School Calc
    Homeroom
    First
    Last
    Grade

    Args:
        file_path (Union[str, os.PathLike]): Path to csv file to import.

    Returns:
        List: List of Asset objects
    """
    assets = []
    with open(file_path, newline='', encoding='utf8') as file:
        reader = csv.DictReader(file)
        for row in reader:

            # Don't sync chromebooks to jamf
            if row['Hardware Type'] == 'Chromebook':
                continue

            # Skip empty rows
            if row['Hardware Type'] is None or row['Hardware Type'] == '':
                continue

            # Skip rows missing serial number and log them
            if row['Serial Number'] is None or row['Serial Number'] == '':
                j_logger.error('Import row skipped, missing Serial Number:')
                j_logger.error('%s \n', row)
                continue

            # Skip rows missing School and log them
            if row['School Calc'] is None or row['School Calc'] == '':
                j_logger.error('Import row skipped, missing School:')
                j_logger.error('%s \n', row)
                continue

            # position is set to empty for iPads
            if row['Hardware Type'] == 'Handheld Device':
                device_type = 'mobile'
                position = ''

            # position is still being used in some jamf policies as a
            # graduation year field so we set that for laptops
            if row['Hardware Type'] == 'Laptop':
                device_type = 'computer'
                position = row['Graduation Year Calc']

            # Assign rows to attributes of asset Object
            asset = Asset(
                device_type=device_type,
                asset_tag=row.get('CCPS Tag#'),
                serial_number=row.get('Serial Number'),
                building=row.get('School Calc'),
                homeroom=row.get('Homeroom'),
                student_name=f"{row.get('First')} {row.get('Last')}",
                student_grade=row.get('Grade'),
                email_address=row.get('Student Email from GAM'),
                grad_year=row.get('Graduation Year Calc'),
                position=position,
                owner='',
            )

            assets.append(asset)
    return assets


def determine_serial_number_key_format(remote_asset: Dict) -> str:
    """
    Determines the format of the serial number key being used by
    Jamf (Classic or Pro) and returns that as a string.

    Args:
        remote_asset (Dict): A Dict obj of a remote asset that was returned
        by Jamf API

    Raises:
        KeyError: Raises a KeyError if neither of the key options was found

    Returns:
        str: String value for the serial number key.
    """
    if remote_asset.get('serialNumber'):
        return 'serialNumber'
    if remote_asset.get('Serial_Number'):
        return 'Serial_Number'
    if remote_asset.get('serial_number'):
        return 'serial_number'
    raise KeyError('No valid key found for Serial Number')



"""
Sets up a log handlers.

Errors are logged in error.log while debugging messages are sent
to the console.
"""

import logging
import os
from dotenv import load_dotenv

load_dotenv()

JAUTOMATE_LOG_LEVEL = os.getenv('JAUTOMATE_LOG_LEVEL', 'INFO')
J_LOG_FILE = os.getenv('J_LOGS')
J_LOG_FILE_LEVEL = os.getenv('J_LOG_FILE_LEVEL', 'ERROR')


def init_logger():
    # Set up logging, debug will output to stream, errors will log to file.
    logger = logging.getLogger("Jautomate")
    logger.setLevel(JAUTOMATE_LOG_LEVEL)

    if J_LOG_FILE:
        logger_file = logging.FileHandler(J_LOG_FILE)
        logger_file.setLevel(J_LOG_FILE_LEVEL)
        error_formater = logging.Formatter(
            '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
        logger_file.setFormatter(error_formater)
        logger.addHandler(logger_file)

    logger_stream = logging.StreamHandler()
    logger_stream.setLevel(JAUTOMATE_LOG_LEVEL)
    debug_formater = logging.Formatter(
        '%(asctime)s - %(name)s - %(message)s',
        '%d-%m-%Y: %H:%M:%S'
    )
    logger_stream.setFormatter(debug_formater)

    logger.addHandler(logger_stream)

    return logger


j_logger = init_logger()



"""Test for jautomate.actions"""
import jautomate.actions


def test_get_building_id():
    building_id = jautomate.actions.get_building_id('test')
    assert building_id is None

    building_id = jautomate.actions.get_building_id('FES')
    assert building_id is not None



"""
These are non cli versions of the commands originally created for use as
a command line tool. Since we moved to calling them from an API there
were issues getting default values passed as the proper type through
the Typer versions in cli.py. These may eventually become the primary
way in which this tool will be used.
"""

from jautomate import actions
from jautomate.assets import Asset, Assets, AssetType
from jautomate.logger import j_logger


def assign_device(
    asset_tag: str,
    building: str,
    device_type: AssetType,
    homeroom: str,
    serial_number: str,
    student_name: str,
    student_grade: str,
    email_address: str = '',
    grad_year: str = '',
    owner: str = '',
    position: str = ''
) -> None:
    """
    Assigns data to a single asset record in Jamf Pro.

    Args:
        device_type (AssetType, required): Type of device being updated. mobile or computer
        asset_tag (str, required): Asset tag should be a 6 digit number
        serial_number (str, required): Serial Number of device
        student_name (str, required):  Student full name
        homeroom (str, required): Homeroom teachers last name
        student_grade (str, required): Student grade as a number
        building (str, required): Three letter building abbreviation. ex: COA
        email_address (Annotated[Optional[str], typer.Option, optional): 
            _description_. Defaults to 'Student email address')]=None.
        grad_year (Annotated[Optional[str], typer.Option, optional): 
            _description_. Defaults to 'Year student graduates')]=None.
        owner (Annotated[Optional[str], typer.Option, optional): 
            _description_. Defaults to 'Owner of device, this was used in the 
            past as a way to target devices but should be deprecated')]=None.
        position (Annotated[Optional[str], typer.Option, optional): 
            _description_. Defaults to 'For now, this will also be the 
            Graduation year, will be transitioned to Student or Staff')]=None.
    """
    j_logger.debug('%s asset assign is running...', device_type.value)

    # Create asset with all passed data
    asset = Asset(
        asset_tag=asset_tag,
        building=building,
        device_type=device_type.value,
        email_address=email_address,
        grad_year=grad_year,
        homeroom=homeroom,
        owner=owner,
        position=position,
        serial_number=serial_number,
        student_grade=student_grade,
        student_name=student_name,
    )

    if asset.device_type == 'computer':
        asset_list = Assets(
            [asset],
            actions.get_single_computer_record_by_asset_tag(asset.asset_tag))
        asset_list = actions.update_computer_jamf_ids_from_asset_tag(
            asset_list)

    if asset.device_type == 'mobile':
        asset_list = Assets([asset], actions.get_all_mobile_devices())
        asset_list = actions.get_mobile_device_jamf_ids_by_serial_number(
            asset_list)

    actions.sync_assets_to_jamf(asset_list.local)


def unassign_device(
    device_type: AssetType,
    asset_tag: str,
) -> None:
    """
    Unassigns data from a single asset record in Jamf Pro

    Args:
        device_type (str, required): Type of device being updated. mobile or computer
        asset_tag (str, required): Asset tag, should be a 6 digit number
    """
    j_logger.debug('%s device unassign is running...', device_type.value)

    # Set all properties to empty str except asset_tag, building, device_type
    asset = Asset(
        asset_tag=asset_tag,
        building='0',
        device_type=device_type.value,
        email_address='',
        grad_year='',
        homeroom='',
        owner='',
        position='',
        phone_number='',
        student_grade='',
        student_name='',
    )

    if device_type.value == 'computer':
        asset_list = Assets(
            [asset],
            actions.get_single_computer_record_by_asset_tag(asset.asset_tag))
        asset_list = actions.update_computer_jamf_ids_from_asset_tag(
            asset_list)

    if asset.device_type == 'mobile':
        asset_list = Assets([asset], actions.get_all_mobile_devices())
        asset_list = actions.get_mobile_device_jamf_ids_by_asset_tag(
            asset_list)

    actions.sync_assets_to_jamf(asset_list.local)



"""Enums for jautomate"""

from enum import Enum


class AssetType(str, Enum):
    """
    Enum to assign asset type in standardized way.

    Used to define cli arguments for various commands and set as part of
    each Asset to determine their type if needed.
    """

    MOBILE = 'mobile'
    COMPUTER = 'computer'
    CHROMEBOOK = 'chromebook'



"""Models for jautomate"""

from pydantic import BaseModel, Field, computed_field

from jautomate.enums import AssetType


class User(BaseModel):
    """User Model"""
    first_name: str
    last_name: str
    email: str
    student_id: int | None = Field(default=None)
    staff_id: str | None = Field(default=None)
    building: str | None = Field(default=None)
    homeroom: str | None = Field(default=None)
    grad_year: str | None = Field(default=None)
    grade: str | None = Field(default=None)
    phone_number: str | None = Field(default=None)
    position: str | None = Field(default=None)
    floor: str | None = Field(default=None)
    owner: str | None = Field(default=None)
    username: str | None = Field(default=None)

    @computed_field
    @property
    def full_name(self) -> str:
        return f"{self.first_name} {self.last_name}"


class Asset(BaseModel):
    """Asset Model"""
    serial_number: str
    asset_tag: str
    device_type: AssetType
    jamf_id: str | None = Field(default=None)
    user: User | None = Field(default=None)
